// CopyRight Philipos Sakellaropoulos 2001
// AsyncDownloader.cpp : Implementation of CAsyncDownloader
#include "stdafx.h"
#include "MyInetUtils.h"
#include "AsyncDownloader.h"

/////////////////////////////////////////////////////////////////////////////
// CAsyncDownloader

STDMETHODIMP CAsyncDownloader::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IAsyncDownloader
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

	//OnData will be used as a callback functiOn 
	//OnData will be called periodically with data from the asynchronous transfer
void CAsyncDownloader::OnData(long DownloadID, long newBytes, long TotalBytes)
{
	CComObject<CDownloadHost<CAsyncDownloader> > *pHost = m_mapID2Host[DownloadID];
	m_mapID2TotalBytes[DownloadID] = TotalBytes;

	ATLTRACE(_T("OnData for ID=%d called with %d bytes\n"),DownloadID,newBytes);

	if(pHost->m_bStarted) 
	{
		long MaxBytes = m_mapID2MaxBytes[DownloadID];
		if(pHost->m_bFirstProgress == false)
		{
			Fire_OnStart(DownloadID, pHost->m_strURL,MaxBytes);
			pHost->m_bFirstProgress = true;
		}
		Fire_OnData(DownloadID,pHost->m_strURL, newBytes, TotalBytes,MaxBytes);
		m_mapID2Status[DownloadID] = DOWNLOAD_INPROGRESS;
	}
}

void CAsyncDownloader::OnFinish(long DownloadID, long TotalBytes)
{
	CComObject<CDownloadHost<CAsyncDownloader> > *pHost = m_mapID2Host[DownloadID];
//	m_mapID2DBytes[DownloadID] = TotalBytes;

	ATLTRACE(_T("OnFinished called for ID=%d\n"),DownloadID); 

	m_mapID2Status[DownloadID] = DOWNLOAD_FINISHED;
	pHost->m_bFinished = true;
	if(pHost->m_bStarted) 
	{
        CProxyIAsyncDownloaderStartEvents<CAsyncDownloader>::Fire_OnFinish(DownloadID,pHost->m_strURL,TotalBytes);
		//pHost->Release();
		if(m_bInDestroying==false)
		{
//			m_mapURL2ID.erase(pHost->m_strURL);	m_mapID2Host.erase(DownloadID);
			m_nDownloading--;
			StartFirstPendingDownload();
		}
	}
}

HRESULT CAsyncDownloader::OnFinalRelease()
{
	Long2HostMap::iterator mapIt;
	CComObject<CDownloadHost<CAsyncDownloader> > *pHost;

	m_bInDestroying = true;
  for(mapIt = m_mapID2Host.begin(); mapIt != m_mapID2Host.end(); mapIt++)
	{
		pHost = (*mapIt).second;
		pHost->AbortDownload();
  }
	while(GarbageCollect()) ;

	m_mapURL2ID.clear();	m_mapID2Host.clear();
  return S_OK;
}

CAsyncDownloader::~CAsyncDownloader()
{
  ATLTRACE("Deleting downloader object\n");
	OnFinalRelease();
}

HRESULT CAsyncDownloader::StartFirstPendingDownload()
{
	HRESULT hr;
	if(qPendingDownloads.empty()) 
		return S_OK;
	if(m_nDownloading < m_ConcurrentDownloads)
	{
		hr = StartDownload(qPendingDownloads.front() );
		qPendingDownloads.pop();
	}
	return hr;
}

void CAsyncDownloader::SetProgressMax(long DownloadID,long ulProgressMax)
{
	m_mapID2MaxBytes[DownloadID] = ulProgressMax;
}

bool CAsyncDownloader::GarbageCollect()
{

	// make a "garbage collection" to kill unneeded hosts
	// cannot kill host in onFinish, the program crashes
	CComObject<CDownloadHost<CAsyncDownloader> > *pHost;
	Long2HostMap::iterator mapIt;
		//pHost->Release(); // destroy host
  for(mapIt = m_mapID2Host.begin(); mapIt != m_mapID2Host.end(); mapIt++)
	{
		pHost = mapIt->second; 
		if(pHost->m_bFinished == true)
		{
			m_mapURL2ID.erase(pHost->m_strURL);	m_mapID2Host.erase(mapIt);
			pHost->Release(); // destroy host
			return true;
			break;
		}
  } 
	return false;
}

HRESULT CAsyncDownloader::StartDownload(long DownloadID)
{
	HRESULT hr;
	CComObject<CDownloadHost<CAsyncDownloader> > *pHost = m_mapID2Host[DownloadID];
	ATLASSERT(pHost != NULL);
	m_mapID2Status[DownloadID] = DOWNLOAD_INPROGRESS;
	hr = pHost->StartDownload();
	pHost->m_bStarted = true;
	if(FAILED(hr) || pHost->m_bFinished) 
	{
 		m_mapURL2ID.erase(pHost->m_strURL);	m_mapID2Host.erase(DownloadID);
		pHost->m_bNoReleaseCallback = true;
		pHost->Release(); // destroy host
  }
	else
		m_nDownloading++;

	if(FAILED(hr)) m_mapID2Status[DownloadID] = DOWNLOAD_ERROR;
	return hr;
}

STDMETHODIMP CAsyncDownloader::RequestDownload(BSTR URL,VARIANT_BOOL Force, long *DownloadID)
{
  long ID; CComBSTR strURL(URL);
	HRESULT hr = S_OK;

	CComObject<CDownloadHost<CAsyncDownloader> > *pHost;
	BSTR2longMap::iterator mapIterator;
  mapIterator = m_mapURL2ID.find(strURL);

	while(GarbageCollect()) ;

  if(mapIterator != m_mapURL2ID.end() )    // found
	{	
      *DownloadID = (*mapIterator).second;
			return S_OK;
	}
	// if we are here no downloading with this URL exists
	hr = CComObject<CDownloadHost<CAsyncDownloader> >::CreateInstance(&pHost);
	if (FAILED(hr))		return hr;
	pHost->AddRef();	//ref.counter becomes 1

	ID = ++m_DownloadCounter;
	pHost->m_pT = this; 
    pHost->m_strURL = URL;
    pHost->m_pFunc1 = &CAsyncDownloader::OnData; 
    pHost->m_pFunc2 = &CAsyncDownloader::OnFinish; 
	pHost->m_DownloadID = ID;
  m_mapURL2ID[strURL] = ID;	m_mapID2Host[ID] = pHost;

	if(m_nDownloading < m_ConcurrentDownloads || Force)
	{
    hr = StartDownload(ID); 
	}
	else {
		qPendingDownloads.push(ID);
		m_mapID2Status[ID] = DOWNLOAD_PENDING;
	}

	*DownloadID = ID;
	return S_OK;
}

STDMETHODIMP CAsyncDownloader::get_ConcurrentDownloads(long *pVal)
{
	*pVal = m_ConcurrentDownloads;
	return S_OK;
}

STDMETHODIMP CAsyncDownloader::put_ConcurrentDownloads(long newVal)
{
	if(newVal < 1)
		return Error("Concurrent downloads number must be greater then zero");

	m_ConcurrentDownloads = newVal;
	return S_OK;
}

STDMETHODIMP CAsyncDownloader::get_DownloadedBytes(long DownloadID, long *pVal)
{
	*pVal = m_mapID2TotalBytes[DownloadID];

	return S_OK;
}

STDMETHODIMP CAsyncDownloader::get_Status(long DownloadID,DownloadStatus *pVal)
{
	*pVal = m_mapID2Status[DownloadID];
	return S_OK;
}

STDMETHODIMP CAsyncDownloader::get_MaxBytes(long DownloadID, long *pVal)
{
	*pVal = m_mapID2MaxBytes[DownloadID]; // map IDs to max bytes
	return S_OK;
}

STDMETHODIMP CAsyncDownloader::AbortDownload(long DownloadID)
{
	CComObject<CDownloadHost<CAsyncDownloader> > *pHost = m_mapID2Host[DownloadID];
	ATLASSERT(pHost != NULL);
	m_mapID2Status[DownloadID] = DOWNLOAD_ABORTED;
	HRESULT hr = pHost->AbortDownload();
	return S_OK;
}

STDMETHODIMP CAsyncDownloader::SuspendDownload(long DownloadID)
{
	CComObject<CDownloadHost<CAsyncDownloader> > *pHost = m_mapID2Host[DownloadID];
	ATLASSERT(pHost != NULL);
	m_mapID2Status[DownloadID] = DOWNLOAD_SUSPENDED;
	HRESULT hr = pHost->SuspendDownload();
	return S_OK;
}

STDMETHODIMP CAsyncDownloader::ResumeDownload(long DownloadID)
{
	CComObject<CDownloadHost<CAsyncDownloader> > *pHost = m_mapID2Host[DownloadID];
	ATLASSERT(pHost != NULL);
	HRESULT hr = pHost->ResumeDownload();
	return S_OK;
}
