// CopyRight Philipos Sakellaropoulos 2001
// AsyncDownloader.h : Declaration of the CAsyncDownloader

#ifndef __ASYNCDOWNLOADER_H_
#define __ASYNCDOWNLOADER_H_

#include "resource.h"       // main symbols
#include "DownloadHost.h"

#include <map>
#include <queue>

#include "MyInetUtils.h"
#include "_IAsyncDownloaderEvents_CP.H"
#include "IAsyncDownloaderStartEvents_CP.H"


/////////////////////////////////////////////////////////////////////////////
// CAsyncDownloader
class ATL_NO_VTABLE CAsyncDownloader : 
    public CComObjectRootEx<CComSingleThreadModel>,
    public CComCoClass<CAsyncDownloader, &CLSID_AsyncDownloader>,
    public ISupportErrorInfo,
    public IConnectionPointContainerImpl<CAsyncDownloader>,
    public IDispatchImpl<IAsyncDownloader, &IID_IAsyncDownloader, &LIBID_MYINETUTILSLib>,
    public CProxy_IAsyncDownloaderEvents< CAsyncDownloader >,
    public CProxyIAsyncDownloaderStartEvents<CAsyncDownloader>,
    public IDispatchImpl<_IAsyncDownloaderEvents, &__uuidof(_IAsyncDownloaderEvents), &LIBID_MYINETUTILSLib, /* wMajor = */ 1, /* wMinor = */ 0>,
    public IDispatchImpl<IAsyncDownloaderStartEvents, &__uuidof(IAsyncDownloaderStartEvents), &LIBID_MYINETUTILSLib, /* wMajor = */ 1, /* wMinor = */ 0>
{
    typedef std::map<CComBSTR,long> BSTR2longMap;
    typedef std::map<long,long> Long2longMap;
    typedef std::map<long,DownloadStatus> Long2StatusMap;
    typedef std::map<long,CComObject<CDownloadHost<CAsyncDownloader> > * > Long2HostMap;
    typedef std::queue<long> QueueOfLongs;

public:
    CAsyncDownloader() : 
      m_ConcurrentDownloads(100),m_nDownloading(0),m_DownloadCounter(0),
          m_bInDestroying(false)
      {
      }

      ~CAsyncDownloader(); 


      DECLARE_REGISTRY_RESOURCEID(IDR_ASYNCDOWNLOADER)

      DECLARE_PROTECT_FINAL_CONSTRUCT()

      BEGIN_COM_MAP(CAsyncDownloader)
          COM_INTERFACE_ENTRY(IAsyncDownloader)
          COM_INTERFACE_ENTRY2(IDispatch, _IAsyncDownloaderEvents)
          COM_INTERFACE_ENTRY(ISupportErrorInfo)
          COM_INTERFACE_ENTRY(IConnectionPointContainer)
          COM_INTERFACE_ENTRY_IMPL(IConnectionPointContainer)
          COM_INTERFACE_ENTRY(_IAsyncDownloaderEvents)
          COM_INTERFACE_ENTRY(IAsyncDownloaderStartEvents)
      END_COM_MAP()
      BEGIN_CONNECTION_POINT_MAP(CAsyncDownloader)
          CONNECTION_POINT_ENTRY(__uuidof(IAsyncDownloaderStartEvents))
          CONNECTION_POINT_ENTRY(__uuidof(_IAsyncDownloaderEvents))
          //CONNECTION_POINT_ENTRY(DIID__IAsyncDownloaderEvents)
      END_CONNECTION_POINT_MAP()


      // ISupportsErrorInfo
      STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid);

      // IAsyncDownloader
public:
    STDMETHOD(ResumeDownload)(/*[in]*/ long DownloadID);
    STDMETHOD(SuspendDownload)(/*[in]*/ long DownloadID);
    STDMETHOD(AbortDownload)(/*[in]*/ long DownloadID);
    STDMETHOD(get_MaxBytes)(/*[in]*/ long DownloadID, /*[out, retval]*/ long *pVal);
    HRESULT OnFinalRelease();
    bool GarbageCollect();
    STDMETHOD(get_Status)(/*[in]*/ long DownloadID,/*[out, retval]*/ DownloadStatus *pVal);
    HRESULT StartFirstPendingDownload();
    STDMETHOD(get_DownloadedBytes)(/*[in]*/ long DownloadID, /*[out, retval]*/ long *pVal);
    HRESULT OnFinalConstruct();
    STDMETHOD(get_ConcurrentDownloads)(/*[out, retval]*/ long *pVal);
    STDMETHOD(put_ConcurrentDownloads)(/*[in]*/ long newVal);
    STDMETHOD(RequestDownload)(/*[in]*/ BSTR URL,VARIANT_BOOL Force,/*[out,retval]*/ long *DownloadID);
    HRESULT StartDownload(long DownloadID);
    void OnData(long DownloadID, long newBytes, long TotalBytes);
    void OnFinish(long DownloadID, long TotalBytes);
    void SetProgressMax(long DownloadID,long ulProgressMax);

protected:
    long m_nDownloading;      // how many files are being downloaded
    long m_DownloadCounter;   // gives the download ID
    BSTR2longMap m_mapURL2ID; // map URL to download IDs
    Long2HostMap m_mapID2Host; // map IDs to Hosts
    Long2longMap m_mapID2TotalBytes; // map IDs to downloaded bytes
    Long2longMap m_mapID2MaxBytes; // map IDs to max bytes bytes
    Long2StatusMap m_mapID2Status; // map IDs to status
    QueueOfLongs qPendingDownloads; // queue of pending downloads

    long m_ConcurrentDownloads;
    bool m_bInDestroying;

    // _IAsyncDownloaderEvents Methods
public:
    STDMETHOD_(void, OnStart)( long DownloadID,  BSTR URL,  long MaxBytes)
    {
        // Add your function implementation here.

    }
    STDMETHOD_(void, OnData)( long DownloadID,  BSTR URL,  long NewBytes,  long TotalBytes,  long MaxBytes)
    {
        // Add your function implementation here.

    }
    STDMETHOD_(void, OnFinish)( long DownloadID,  BSTR URL,  long TotalBytes)
    {
        // Add your function implementation here.

    }

    // IAsyncDownloaderStartEvents Methods
    STDMETHOD_(HRESULT, Finished)( long DownloadID,  BSTR URL,  long TotalBytes)
    {
        // Add your function implementation here.
        return E_NOTIMPL;
    }   

public:
};

#endif //__ASYNCDOWNLOADER_H_
