// (C) Philip Sakellaropoulos 2001
// DownloadHost.h

#ifndef __DOWNLOADHOST_H_
#define __DOWNLOADHOST_H_

#include "resource.h"       // main symbols

// derive from ATL class CBindStatusCallback
// change GetBindInfo indicate that we want writing to cache
template <class T>
class ATL_NO_VTABLE CMyBindStatusCallback : public CBindStatusCallback<T>
{
	public:
		T* m_pT;

	private:
		bool _bProgressMaxSet;

	public:

	CMyBindStatusCallback()
	{ _bProgressMaxSet = false; }

	HRESULT OnFinalRelease()
	{
		ATLTRACE("Final release for CMyBindStatusCallback\n");
	}
	// copied from ATLCTL.H, i removed  BINDF_GETNEWESTVERSION,BINDF_NOWRITECACHE flag
	STDMETHOD(GetBindInfo)(DWORD *pgrfBINDF, BINDINFO *pbindInfo)
	{
		ATLTRACE(_T("CBindStatusCallback::GetBindInfo\n"));

		if (pbindInfo==NULL || pbindInfo->cbSize==0 || pgrfBINDF==NULL)
			return E_INVALIDARG;

		*pgrfBINDF = BINDF_ASYNCHRONOUS | BINDF_ASYNCSTORAGE; 
			//| BINDF_GETNEWESTVERSION; // | BINDF_NOWRITECACHE;

		ULONG cbSize = pbindInfo->cbSize;		// remember incoming cbSize
		memset(pbindInfo, 0, cbSize);			// zero out structure
		pbindInfo->cbSize = cbSize;				// restore cbSize
		pbindInfo->dwBindVerb = BINDVERB_GET;	// set verb
		return S_OK;
	}
	STDMETHOD(OnProgress)(ULONG ulProgress, ULONG ulProgressMax, ULONG ulStatusCode, LPCWSTR szStatusText)
	{
		ATLTRACE(_T("OnProgress,ulProgressMax=%d,ulStatusCode=%d\n"),ulProgressMax,ulStatusCode);
		if(ulProgressMax>0 && !_bProgressMaxSet) {
			m_pT->SetProgressMax(ulProgressMax);
			_bProgressMaxSet = true;
		}
		return S_OK;
	}
};


template <class T>
class ATL_NO_VTABLE CDownloadHost :
	public CComObjectRootEx<CComSingleThreadModel>
{
	typedef void (T::*PMYFUNCDOWNLOADDATA)(long , long , long );
	typedef void (T::*PMYFUNCFINISHDATA)(long, long);
	typedef CMyBindStatusCallback<CDownloadHost<T> >  CMyCallback;

 public:
	CDownloadHost() : 
		m_TotalBytes(0),m_pFunc1(NULL),m_pFunc2(NULL),m_pT(NULL),
			m_bStarted(false),m_bFinished(false),m_bFirstProgress(false),
			m_bNoReleaseCallback(false)
	{	}

	~CDownloadHost() { ATLTRACE("Deleting host with ID=%d\n",m_DownloadID); }

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CDownloadHost)
END_COM_MAP()

	//OnData will be used as a callback function by the CBindStatusCallback object.
	//OnData will be called periodically with data from the asynchronous transfer
	void OnData(CBindStatusCallback<CDownloadHost<T> >*
	     pbsc, BYTE* pBytes, DWORD dwSize)
	{
		m_TotalBytes += dwSize;

		if(pBytes != NULL) // progress data
			(m_pT->*m_pFunc1)(m_DownloadID, dwSize, m_TotalBytes);
    else  // finished data
			(m_pT->*m_pFunc2)(m_DownloadID, m_TotalBytes);
	}
	
	void SetProgressMax(long ulProgressMax)
	{
		m_pT->SetProgressMax(m_DownloadID,ulProgressMax);
	}

	HRESULT StartDownload()
	{
		//return CBindStatusCallback<CDownloadHost<T> >::Download(this, OnData, m_strURL, NULL, FALSE);
		return m_pbsc->StartAsyncDownload(this, &CDownloadHost<T>::OnData, m_strURL, NULL, FALSE);
	}
  HRESULT FinalConstruct()
	{
		HRESULT hRes = CComObject<CMyCallback>::CreateInstance(&m_pbsc);
		m_pbsc->AddRef();
		m_pbsc->m_pT = this;
		if (FAILED(hRes))		return hRes;

		return S_OK;
	}
  HRESULT FinalRelease()
	{
		long nC = 0;
		if(m_bNoReleaseCallback == false)
			nC = m_pbsc->Release();	
		ATLTRACE("Releasing callback returned %d --------- \n",nC);
		return S_OK;
	}
	HRESULT AbortDownload()
	{
		if(m_bFinished==true || m_pbsc->m_spBinding == NULL)
			return S_OK;
		return m_pbsc->m_spBinding->Abort();
	}
	HRESULT SuspendDownload()
	{
		if(m_bFinished==true || m_pbsc->m_spBinding == NULL)
			return S_OK;
		return m_pbsc->m_spBinding->Suspend();
	}
	HRESULT ResumeDownload()
	{
		if(m_bFinished==true || m_pbsc->m_spBinding == NULL)
			return S_OK;
		return m_pbsc->m_spBinding->Resume();
	}
 public:
	long m_TotalBytes;
	long m_DownloadID;
	bool m_bStarted,m_bFinished,m_bFirstProgress;
	bool m_bNoReleaseCallback; // after many experiments
	T *m_pT;
	PMYFUNCDOWNLOADDATA m_pFunc1;
	PMYFUNCFINISHDATA m_pFunc2;
	CComBSTR m_strURL;
	CComObject<CMyCallback > *m_pbsc;
};



#endif //__DOWNLOADHOST_H_
